@@ -49,7 +49,7 @@ def get_channel(e):
 
 def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis.fits', 
                     in_level_down=0, in_level_up = 1e10, quantile=0.8, make_plot=True, 
-                    far=1e-4, histo_bins=300):
+                    far=1e-4, histo_bins=300, time0=0):
 	"""Creates user GTI based on hisogram of rates
 
 	Args:
@@ -64,6 +64,7 @@ def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis
 		make_plot (bool, optional): if making plots. Defaults to True.
 		far (_type_, optional): the False Alarm Rate to exclude intervals from the fitted distribution. Defaults to 1e-4.
 		histo_bins (int, optional): number of bins in the histogram. Defaults to 300.
+		time0: a time to add to GTI times to be consistnt with observations
 
 	Returns:
 		_type_: _description_
@@ -96,8 +97,8 @@ def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis
 	new_gti = create_gti_from_condition(time, (rate > limit_down) & (rate < limit_up) )
 
 	if len(new_gti.shape)>1:
-		start=new_gti[:,0]
-		stop=new_gti[:,1]
+		start=new_gti[:,0]+time0
+		stop=new_gti[:,1]+time0
 		c1 = pf.Column(name='START', format='D', unit='s', array=start)
 		c2 = pf.Column(name='STOP', format='D', unit='s', array=stop)
 		coldefs = pf.ColDefs([c1, c2])
