@@ -306,9 +306,24 @@ y
 '''
 
 
+xselect_gti_filter='''dummy
+read eve %s
 
 
-def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0, t_ref=0):
+filter time file %s
+extract events
+save events %s
+yes
+quit
+no
+
+'''
+
+#event_file_name  gti_file_name  output_event_file_name
+
+
+
+def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0, t_ref=0, user_gti=None):
     
     '''
     Wrapper on timingsuite to produce Energy - Phase matrixes for source and background for both FPM units.
@@ -320,6 +335,7 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
     :param orbitfile: orbitfile for orbital correction (timingsuite) if None
     :param nudot: spinn requency derivative (=0)
     :param t_ref: reference time for folding (if <= 0, it takes the first time)
+    ;param user_gti : user GTI file (defaults to None to avoid extraction)
     :return: it writes the fits files sourceA_ENPHASE.fits sourceB_ENPHASE.fits
                         backgroundA_ENPHASE.fits backgroundB_ENPHASE.fits
     '''
@@ -327,7 +343,13 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
     for tt in ['source', 'background']:
         for unit in ['A', 'B']:
             with open('list_evt.txt', 'w') as ff:
-                ff.write('%s%s.evt' % (tt, unit))
+                if user_gti is not None:
+                    with open('gti_select_cmd.txt', 'w') as f2:
+                        f2.write(xselect_gti_filter %('%s%s.evt' % (tt, unit), user_gti, '%s%s_filtered.evt' % (tt, unit) ))
+                    run('xselect < gti_select_cmd.txt')
+                    ff.write('%s%s_filtered.evt' % (tt, unit) )
+                else:
+                    ff.write('%s%s.evt' % (tt, unit))
 
             if orbitfile is None:
                 with open('timing_cmd.txt', 'w') as ff:
@@ -602,7 +624,7 @@ n
 %f
 '''
 
-def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=None, nudot=0, t_ref=0):
+def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=None, nudot=0, t_ref=0, user_gti=None):
     """It builds the time-phase matrix
 
     Args:
@@ -614,6 +636,7 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
         orbitfile (string, optional): name of the orbit correction file. Defaults to None.
         nudot (int, optional): frequency derivative. Defaults to 0.
         t_ref (int, optional): folding reference time (<=0 it takes the first time). Defaults to 0 to take the first time.
+        user_gti (str, optional): file name for user GTI, default None skips the GTI selection
 
     Raises:
         It writes the time phase matrixes in fits format
@@ -621,7 +644,13 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
     for tt in ['source', 'background']:
         for unit in ['A', 'B']:
             with open('list_evt.txt', 'w') as ff:
-                ff.write('%s%s.evt' % (tt, unit))
+                if user_gti is not None:
+                    with open('gti_select_cmd.txt', 'w') as f2:
+                        f2.write(xselect_gti_filter %('%s%s.evt' % (tt, unit), user_gti, '%s%s_filtered.evt' % (tt, unit) ))
+                    run('xselect < gti_select_cmd.txt')
+                    ff.write('%s%s_filtered.evt' % (tt, unit) )
+                else:
+                    ff.write('%s%s.evt' % (tt, unit))
 
             with open('timing_cmd.txt', 'w') as ff:
                 if orbitfile is None:
@@ -825,7 +854,8 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
                               division_derivative_order=2, max_n_high_lines = 2, forced_gaussian_centroids = [],
                               forced_gaussian_amplitudes = [],
                               forced_gaussian_sigmas = [],
-                              y_lim=[0,1]):
+                              y_lim=[0,1], 
+                              noFe=False):
     """This function implements the logics to perform a fit of the pulsed fraction.
     It first interpolates the pulsed fraction with a spline to find extremes of the function.
     Then it performs a fit with a polynomial plus gaussians.
@@ -859,7 +889,8 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
                                                     is different from the one of forced_gaussian_centroids, it is unused. Defaults to [].
         forced_gaussian_sigmas (list, optional): you can force the initial sigmas with this parameter, if the length of the list 
                                                  is different from the one of forced_gaussian_centroids, it is unused. Defaults to [].
-        y_lim : limits for the plotting. DEfaults to [0,1]
+        y_lim : limits for the plotting. Defaults to [0,1]
+        noFe : exclude the 6.4 keV Gaussian feature, defaults to False
     Returns:
         two fitting objects for the two ranges (the output of utils.fit_pulsed_frac). if only one is used, the second is None.
      """    
@@ -908,10 +939,17 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
     if np.sum(ind_range1) <= 0:
         make_double_fit = False
 
+    if noFe:
+        center=[]
+        sigma =[]
+        amplitude=[]
+        n_gauss=0 
+    else:
+        center=[[6.5, 5.5, 7.5]]
+        sigma=[[0.2,0,2.0]]
+        amplitude=[[0,-1,1]]
+        n_gauss=1
 
-    center=[[6.5, 5.5, 7.5]]
-    sigma=[[0.2,0,1.0]]
-    amplitude=[[0,-1,1]]
     if make_double_fit:
         if division_derivative_order == 2:
             e_turn = get_zeros(fake_en, smoothed_pulsed_frac_deriv_2, None, ind_range1)
@@ -934,13 +972,12 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
         ind_low = ee_pulsed <= e_turn
         ind_high = ee_pulsed > e_turn
         logger.info('We use %d points in the low-energy' % np.sum(ind_low))
-        n_gauss = 1
+        
     else:
         #all values
         ind_low = ee_pulsed > 0
         #no value
         ind_high = ee_pulsed > 0
-        n_gauss = 1
 
         if len(forced_gaussian_centroids) > 0:
             center += [ [ee, (1-e_threshold)*ee, (1+e_threshold)*ee] for ee in forced_gaussian_centroids]
@@ -1062,9 +1099,9 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
     ax1.set_title(title)
 
     ax2.set_ylabel('Residuals')
-    ax1.set_ylim(-0.1, 1)
+    #ax1.set_ylim(-0.1, 1)
     if make_double_fit:
-        ax2.errorbar(ee_pulsed, np.concatenate([bb_low,bb_high]), xerr=dee_pulsed, yerr=1., fmt='.', color=col[2])
+        ax2.errorbar(ee_pulsed, np.concatenate([bb_low, bb_high]), xerr=dee_pulsed, yerr=1., fmt='.', color=col[2])
     else:
         ax2.errorbar(ee_pulsed, bb_low, xerr=dee_pulsed, yerr=1., fmt='.', color=col[2])
     ax2.axhline(y=0, color='k', linestyle='--')
@@ -1190,7 +1227,7 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         ax1.set_xlim(x_lim)
         
         ax2.set_ylabel('Residuals')
-        ax1.set_ylim(-0.1, 1)
+        #ax1.set_ylim(-0.1, 1)
         ax2.errorbar(en, bb, xerr=den, yerr=1., fmt='.', color=col[2])
         ax2.axhline(y=0, color='k', linestyle='--')
         plt.savefig(stem + 'pulsed_fitted.pdf')
@@ -1199,6 +1236,9 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
     outputfile.write(out.fit_report())
     outputfile.close()
 
+    with open(stem+'.json', 'w') as jf:
+        out.params.dump(jf)
+
     return out
 
 
@@ -1248,6 +1288,7 @@ def explore_fit_mcmc(model_fitted, pars=[], hm_steps=7000, hm_walkers=2000, hm_j
     if plot_corner:
         emcee_plot = corner.corner(explore.flatchain, labels=explore.var_names, show_titles=True,
                                    plot_datapoints=True)
+        emcee_plot.savefig(stem+'_corner.png')
 
     highest_prob = np.argmax(explore.lnprob)
     hp_loc = np.unravel_index(highest_prob, explore.lnprob.shape)
@@ -1275,6 +1316,8 @@ def explore_fit_mcmc(model_fitted, pars=[], hm_steps=7000, hm_walkers=2000, hm_j
     logger.info('Parameter -3sigma -2sigma -1sigma median +1sigma +2sigma +3sigma')
 
     if print_file:
+        with open(stem+'.json', 'w') as jf:
+            explore.params.dump(jf)
         o_f_emcee = open(stem + '.out', 'w')
         o_f_emcee.write('# Maximum Likelihood Estimation from emcee \n#Parameter MLE Value Median Value Uncertainty \n')
         for name, param in pars2.items():
@@ -1282,18 +1325,18 @@ def explore_fit_mcmc(model_fitted, pars=[], hm_steps=7000, hm_walkers=2000, hm_j
         o_f_emcee.write(
             '#\Error estimates from emcee: \n#Parameter -3sigma -2sigma -1sigma median +1sigma +2sigma +3sigma \n')
 
-    for name in pars2.keys():
-        quantiles = np.percentile(explore.flatchain[name], [0.135, 2.275, 15.865, 50, 84.135, 97.725, 99.865])
-        median = quantiles[3]
-        err_m3 = quantiles[0] - median
-        err_m2 = quantiles[1] - median
-        err_m1 = quantiles[2] - median
-        err_p1 = quantiles[4] - median
-        err_p2 = quantiles[5] - median
-        err_p3 = quantiles[6] - median
-        fmt = '{:5s} {:8.4f} {:8.4f} {:8.4f} {:8.4f} {:8.4f} {:8.4f} {:8.4f}'.format
-        logger.info(fmt(name, err_m3, err_m2, err_m1, median, err_p1, err_p2, err_p3))
-        o_f_emcee.write(fmt(name, err_m3, err_m2, err_m1, median, err_p1, err_p2, err_p3) + '\n')
+        for name in pars2.keys():
+            quantiles = np.percentile(explore.flatchain[name], [0.135, 2.275, 15.865, 50, 84.135, 97.725, 99.865])
+            median = quantiles[3]
+            err_m3 = quantiles[0] - median
+            err_m2 = quantiles[1] - median
+            err_m1 = quantiles[2] - median
+            err_p1 = quantiles[4] - median
+            err_p2 = quantiles[5] - median
+            err_p3 = quantiles[6] - median
+            fmt = '{:5s} {:8.4f} {:8.4f} {:8.4f} {:8.4f} {:8.4f} {:8.4f} {:8.4f}'.format
+            logger.info(fmt(name, err_m3, err_m2, err_m1, median, err_p1, err_p2, err_p3))
+            o_f_emcee.write(fmt(name, err_m3, err_m2, err_m1, median, err_p1, err_p2, err_p3) + '\n')
 
     o_f_emcee.close()
 
@@ -1359,7 +1402,9 @@ def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=-1, plot=Fals
         # print(chi2_sf, old_chi2_sf)
         #Sometimes, we do ot reach the required level, but we cannot describe the pulse significantly better,
         # so we stop in any case (condition chi2_sf < old_chi2_sf)
-        if chi2_sf > level or chi2_sf < old_chi2_sf:
+        if chi2_sf > level or (chi2_sf < old_chi2_sf and chi2_sf > level/100.):
+            if verbose:
+                logger.info('chi2_sf = %e (level is %.2f) old_chi2_sf = %e' % (chi2_sf, level, old_chi2_sf))
             break
         old_chi2_sf = chi2_sf
 
@@ -1538,7 +1583,7 @@ def plot_matrix_as_lines(t, pp, dpp, kind='T', normalize=False, offset=0):
     plt.xlabel('Phase')
     #plt.legend()
 
-def get_fourier_coeff(pulse):
+def get_fourier_coeff(pulse, n=2):
     '''
     returns the first two harmonic coefficients
     :param pulse:
@@ -1549,38 +1594,47 @@ def get_fourier_coeff(pulse):
     a2 relative amplitude of second harmonic
     '''
 
-    phi = np.linspace(0, 2 * np.pi, len(pulse))
-    i_c = np.sum(np.cos(phi) * pulse) / np.pi
-    i_s = np.sum(np.sin(phi) * pulse) / np.pi
-    i_c2 = np.sum(np.cos(2 * phi) * pulse) / np.pi
-    i_s2 = np.sum(np.sin(2 * phi) * pulse) / np.pi
+    # phi = np.linspace(0, 2 * np.pi, len(pulse))
+    # i_c = np.sum(np.cos(phi) * pulse) / np.pi
+    # i_s = np.sum(np.sin(phi) * pulse) / np.pi
+    # i_c2 = np.sum(np.cos(2 * phi) * pulse) / np.pi
+    # i_s2 = np.sum(np.sin(2 * phi) * pulse) / np.pi
 
-    phi0 = np.arctan2(i_s, i_c) / 2 / np.pi
-    phi0_2 = np.arctan2(i_s2, i_c2) / 2 / np.pi
-    a = np.sqrt(i_c * i_c + i_s * i_s) / len(pulse) / np.mean(pulse)
-    a2 = np.sqrt(i_c2 * i_c2 + i_s2 * i_s2) / len(pulse) / np.mean(pulse)
+    # phi0 = np.arctan2(i_s, i_c) / 2 / np.pi
+    # phi0_2 = np.arctan2(i_s2, i_c2) / 2 / np.pi
+    # a = np.sqrt(i_c * i_c + i_s * i_s) / len(pulse) / np.mean(pulse)
+    # a2 = np.sqrt(i_c2 * i_c2 + i_s2 * i_s2) / len(pulse) / np.mean(pulse)
 
-    return phi0, phi0_2, a, a2
+    ff = np.fft.rfft(pulse)
+    if n>len(pulse)/2:
+        raise IndexError('get_fourier_coeff: you asked for too many coefficients %d' %n )
 
-def get_fourier_coeff_error( counts, counts_err, n_simul=100, use_poisson=False):
+    a = np.abs(ff[1:n+1])/ff[0].real
+    phi = np.angle(ff[1:n+1])/2/np.pi
+    return a, phi
 
-    sim_phi0 = []
-    sim_phi0_2 = []
-    sim_A = []
-    sim_A2 = []
+def get_fourier_coeff_error( counts, counts_err, n_simul=1000, use_poisson=False, n=2):
 
+    sim_a = []
+    sim_phi = []
+    
     for i in range(n_simul):
         if use_poisson:
             fp = random.poisson(counts)
         else:
             fp = random.normal(counts, counts_err)
-        phi0, phi0_2, A, A2 = get_fourier_coeff(fp)
-        sim_phi0.append(phi0)
-        sim_phi0_2.append(phi0_2)
-        sim_A.append(A)
-        sim_A2.append(A)
-
-    return np.std(sim_phi0), np.std(sim_phi0_2,), np.std(sim_A), np.std(sim_A2)
+        a , phi = get_fourier_coeff(fp, n=n)
+        sim_phi.append(phi)
+        sim_a.append(a)
+    err_a = np.zeros(n)
+    err_phi = np.zeros(n)
+    for i in range(n):
+        y = [x[i] for x in sim_a ]
+        err_a[i] = np.std(y)
+        y = [x[i] for x in sim_phi ]
+        err_phi[i] = np.std(y)
+        
+    return err_a, err_phi
 
 def rate_filter(time, rate, drate, level_down=0, level_up = 1e10, quantile=0.8, make_plot=True, 
                 far=1e-4, rate_label='rate', histo_bins=300):
